package com.smppw.analysis.client;

import com.smppw.analysis.application.dto.WithSecReq;
import com.smppw.analysis.application.dto.info.*;
import com.smppw.analysis.application.service.info.FundInfoService;
import com.smppw.analysis.application.service.info.GlobalService;
import com.smppw.analysis.domain.dto.info.*;
import com.smppw.common.pojo.ResultVo;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @author wangzaijun
 * @date 2023/8/7 15:40
 * @description 基本信息api
 */
@RestController
@RequestMapping("/v1/api/fund")
public class FundApi {
    private final FundInfoService service;
    private final GlobalService globalService;

    public FundApi(FundInfoService service, GlobalService globalService) {
        this.service = service;
        this.globalService = globalService;
    }

    /**
     * 标的头部信息接口
     *
     * @param params 请求参数
     * @return /
     */
    @PostMapping("head-info")
    public ResultVo<? extends HeadInfoVO> headInfo(@RequestBody HeadInfoReq params) {
        return ResultVo.ok(this.globalService.headInfo(params));
    }

    /**
     * 统一设置接口，返回所有标的的统一设置，包括时段、净值频率等
     *
     * @param params /
     * @return /
     */
    @PostMapping("common-info")
    public ResultVo<CommonInfoVO> commonInfo(@RequestBody CommonInfoReq params) {
        return ResultVo.ok(this.globalService.commonInfo(params));
    }

    /**
     * 头部指标计算接口
     *
     * @param params 请求参数
     * @return /
     */
    @PostMapping("head-indicator")
    public ResultVo<HeadIndicatorVO> headIndicator(@RequestBody HeadIndicatorReq params) {
        return ResultVo.ok(this.globalService.headIndicator(params));
    }

    /**
     * 相似产品（私募基金）
     *
     * @param params /
     * @return /
     */
    @PostMapping("similar")
    public ResultVo<List<FundSimilarVO>> similar(@RequestBody FundSimilarReq params) {
        return ResultVo.ok(this.service.getFundSimilarList(params));
    }

    /**
     * 公募基金公告
     *
     * @param params 基金id
     * @return 基金公告
     */
    @PostMapping("notice-info")
    public ResultVo<List<ManualFundNoticeInfoVO>> noticeInfo(@RequestBody ManualFundNoticeReq params) {
        return ResultVo.ok(service.noticeList(params));
    }

    /**
     * 公募基金投资类信息说明
     *
     * @return 投资类信息说明
     */
    @PostMapping("invest-info")
    public ResultVo<ManualFundInvestInfoVO> investInfo(@RequestBody WithSecReq req) {
        return ResultVo.ok(service.investInfo(req.getSecId()));
    }

    /**
     * 私募基金费率信息
     *
     * @return 基金费率信息
     */
    @PostMapping("fee")
    public ResultVo<FundBaseFeeVO> fee(@RequestBody WithSecReq req) {
        return ResultVo.ok(this.service.getFundFee(req.getSecId()));
    }

    /**
     * 公募基金费率信息
     *
     * @return 基金费率信息
     */
    @PostMapping("mf-fee")
    public ResultVo<ManualFundFeeInfoVO> mFFundFee(@RequestBody WithSecReq req) {
        return ResultVo.ok(this.service.getMfFundFee(req.getSecId()));
    }


    /**
     * 私募基金详情页-基金经理
     *
     * @return 基金经理信息
     */
    @PostMapping("manager-info")
    public ResultVo<List<FundManagerInfoVo>> managerInfo(@RequestBody WithSecReq req) {
        return ResultVo.ok(this.service.getHFManagerInfo(req.getSecId()));
    }

    /**
     * 公募基金详情页-基金经理
     *
     * @param params 详情请看请求体
     * @return 基金经理信息
     */
    @PostMapping("mf-manager-info")
    public ResultVo<List<ManualFundManagerInfoVO>> managerInfo(@RequestBody ManualFundManagerParams params) {
        return ResultVo.ok(this.service.managerList(params));
    }

    /**
     * 公募基金的基金经理变更历史
     *
     * @param params 详情请看请求体
     * @return 公募基金的基金经理变更历史
     */
    @PostMapping("manager-change")
    public ResultVo<List<ManualFundManagerChangeVO>> managerChange(@RequestBody ManualFundManagerParams params) {
        return ResultVo.ok(this.service.managerChangeList(params));
    }

    /**
     * 私募基金的基本信息
     *
     * @return 私募基金基本信息
     */
    @PostMapping("base-info")
    public ResultVo<PrivatelyFundBaseInfoVO> getFundBaseInfo(@RequestBody WithSecReq req) {
        return ResultVo.ok(this.service.getFundBaseInfo(req.getSecId()));
    }

    /**
     * 公募基金的基本信息
     *
     * @return 私募基金基本信息
     */
    @PostMapping("mf-base-info")
    public ResultVo<PubliclyFundBaseInfoVO> getMfFundBaseInfo(@RequestBody WithSecReq req) {
        return ResultVo.ok(this.service.getMfFundBaseInfo(req.getSecId()));
    }
}
