package com.smppw.analysis.client;

import com.smppw.analysis.application.dto.BaseReq;
import com.smppw.analysis.application.dto.position.*;
import com.smppw.analysis.application.service.position.FundFuturesOptionService;
import com.smppw.analysis.application.service.position.FuturePositionAnalysis;
import com.smppw.analysis.application.service.position.StockPositionAnalysis;
import com.smppw.analysis.application.service.position.SynthesizePositionAnalysis;
import com.smppw.analysis.domain.dto.position.bond.BondSortAllocationParam;
import com.smppw.analysis.domain.dto.position.stock.ChangeNumberVO;
import com.smppw.analysis.domain.dto.position.stock.MajorChangeVO;
import com.smppw.analysis.domain.dto.position.synthesize.HolderInfoVO;
import com.smppw.analysis.domain.dto.position.synthesize.LeverageChangeVO;
import com.smppw.analysis.domain.dto.position.synthesize.PositionInfoVO;
import com.smppw.analysis.domain.dto.position.synthesize.PositionListVO;
import com.smppw.analysis.domain.manager.position.bond.BondPositionService;
import com.smppw.common.pojo.ResultVo;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * @author wangzaijun
 * @date 2023/6/6 11:23
 * @description 基金 持仓分析，包括公募、私募基金，区分综合、股票、债券和期货tab组件
 */
@RestController
@RequestMapping("/v1/api/position")
public class FundPositionApi {
    private final StockPositionAnalysis stock;
    private final FuturePositionAnalysis future;
    private final SynthesizePositionAnalysis synthesize;
    private final BondPositionService bondPositionService;
    private final FundFuturesOptionService fundFuturesOptionService;

    public FundPositionApi(StockPositionAnalysis stock, FuturePositionAnalysis future,
                           SynthesizePositionAnalysis synthesize, BondPositionService bondPositionService,
                           FundFuturesOptionService fundFuturesOptionService) {
        this.stock = stock;
        this.future = future;
        this.synthesize = synthesize;
        this.bondPositionService = bondPositionService;
        this.fundFuturesOptionService = fundFuturesOptionService;
    }

    /**
     * 公募私募，综合-资产大类配置
     *
     * @param params /
     * @return /
     */
    @PostMapping("asset-allocation")
    public ResultVo<Map<String, Object>> getAssetAllocation(@RequestBody AssetAllocationReq params) {
        return ResultVo.ok(this.synthesize.getAssetAllocation(params));
    }

    /**
     * 公募私募，综合-杠杆变化
     *
     * @param params /
     * @return /
     */
    @PostMapping("leverage-change")
    public ResultVo<List<LeverageChangeVO>> getLeverageChange(@RequestBody LeverageChangeReq params) {
        return ResultVo.ok(this.synthesize.getLeverageChange(params));
    }

    /**
     * 公募私募，综合-持仓列表的请求参数
     *
     * @param params /
     * @return /
     */
    @PostMapping("position-param")
    public ResultVo<PositionInfoVO> getPositionParams(@RequestBody PositionInfoReq params) {
        return ResultVo.ok(this.synthesize.getPositionParams(params));
    }

    /**
     * 公募私募，综合-持仓列表
     *
     * @param params /
     * @return /
     */
    @PostMapping("position-list")
    public ResultVo<List<PositionListVO>> getPosition(@RequestBody PositionListReq params) {
        return ResultVo.ok(this.synthesize.getPosition(params));
    }

    /**
     * 公募，综合-持有人结构
     *
     * @param params /
     * @return /
     */
    @PostMapping("holder-info")
    public ResultVo<List<HolderInfoVO>> getHolderInfo(@RequestBody HolderInfoReq params) {
        return ResultVo.ok(this.synthesize.getHolderInfo(params));
    }

    /**
     * 公募，股票-持仓股票重大变动
     *
     * @param params /
     * @return /
     */
    @PostMapping("stock/major-change")
    public ResultVo<List<MajorChangeVO>> getMajorChange(@RequestBody MajorChangeReq params) {
        return ResultVo.ok(this.stock.getMajorChangeList(params));
    }

    /**
     * 公募私募，股票-集中度
     *
     * @param params /
     * @return /
     */
    @PostMapping("stock/crn")
    public ResultVo<Map<String, Object>> getStockConcentration(@RequestBody ConcentrationReq params) {
        return ResultVo.ok(this.stock.getConcentration(params));
    }

    /**
     * 公募私募，股票-持股数量变动时序
     *
     * @param params /
     * @return /
     */
    @PostMapping("stock/change-num")
    public ResultVo<List<ChangeNumberVO>> getStockChangeNumber(@RequestBody ChangeNumberReq params) {
        return ResultVo.ok(this.stock.getChangeNumber(params));
    }

    /**
     * 公募私募，股票-行业配置
     *
     * @param params /
     * @return /
     */
    @PostMapping("stock/industry-allocation")
    public ResultVo<Map<String, Object>> getStockIndustryAllocation(@RequestBody StockAllocationReq params) {
        return ResultVo.ok(this.stock.getIndustryAllocation(params));
    }

    /**
     * 公募私募，股票-行业配置偏好时序
     *
     * @param params /
     * @return /
     */
    @PostMapping("stock/industry-allocation-preference")
    public ResultVo<Map<String, Object>> getStockIndustryAllocationPreference(@RequestBody StockAllocationReq params) {
        return ResultVo.ok(this.stock.getIndustryAllocationPreference(params));
    }

    /**
     * 公募私募，股票-风格配置
     *
     * @param params /
     * @return /
     */
    @PostMapping("stock/style-allocation")
    public ResultVo<Map<String, Object>> getStockStyleAllocation(@RequestBody StockAllocationReq params) {
        return ResultVo.ok(this.stock.getStyleAllocation(params));
    }

    /**
     * 公募私募，股票-流动性分析
     *
     * @param params /
     * @return /
     */
    @PostMapping("stock/liquidity-allocation")
    public ResultVo<Map<String, Object>> getStockLiquidityAllocation(@RequestBody StockAllocationReq params) {
        return ResultVo.ok(this.stock.getLiquidityAllocation(params));
    }

    /**
     * 公募私募，股票-业绩归因
     *
     * @param params /
     * @return /
     */
    @PostMapping("stock/performance-attribution")
    public ResultVo<Map<String, Object>> getStockPerformanceAttribution(@RequestBody StockPerformanceAttributionReq params) {
        return ResultVo.ok(this.stock.getPerformanceAttribution(params));
    }

    /**
     * 公募私募，股票-Barra敏感度分析
     *
     * @param params /
     * @return /
     */
    @PostMapping("stock/barra-sensitivity")
    public ResultVo<Map<String, Object>> getStockBarraSensitivity(@RequestBody BarraSensitivityReq params) {
        return ResultVo.ok(this.stock.getBarraSensitivity(params));
    }

    /**
     * 债券分类配置及明细  公募私募
     *
     * @param param /
     * @return /
     */
    @PostMapping("bond/sort-allocation")
    public ResultVo<Map<String, Object>> getBondSortAllocation(@RequestBody BondSortAllocationParam param) {
        return ResultVo.ok(bondPositionService.getBondSortAllocation(param));
    }

    /**
     * 债券集中度
     *
     * @param param /
     * @return /
     */
    @PostMapping("bond/crn")
    public ResultVo<Map<String, Object>> getBondConcentration(@RequestBody BondSortAllocationParam param) {
        return ResultVo.ok(bondPositionService.getBondConcentration(param));
    }

    /**
     * 债券信用评级
     *
     * @param param /
     * @return /
     */
    @PostMapping("/bond/credit-grading")
    public ResultVo<Map<String, Object>> getBondCreditGrading(@RequestBody BondSortAllocationParam param) {
        return ResultVo.ok(bondPositionService.getBondCreditGrading(param));
    }

    /**
     * @return
     */
    @PostMapping(value = "/fund-futures-option")
    public ResultVo<Map<String, Object>> fundFuturesOption(@RequestBody Map<String, Object> params) {
        return fundFuturesOptionService.fundFuturesOption(params);
    }


    /**
     * @return
     */
    @PostMapping(value = "/fund-futures-option-breed")
    public ResultVo<Map<String, Object>> fundFuturesOptionForBreed(@RequestBody Map<String, Object> params) {
        return fundFuturesOptionService.fundFuturesOptionForBreed(params);
    }

    /**
     * 板块持仓分析
     *
     * @return
     */
    @PostMapping(value = "/fund-position-analysis")
    public ResultVo<Map<String, Object>> fundPositionAnalysis(@RequestBody Map<String, Object> params) {
        return fundFuturesOptionService.fundPositionAnalysis(params);
    }

    /**
     * 板块持仓分析
     *
     * @return
     */
    @PostMapping(value = "/fund-position-analysis-line")
    public ResultVo<Map<String, Object>> fundPositionAnalysisLine(@RequestBody Map<String, Object> params) {
        return fundFuturesOptionService.fundPositionAnalysisLine(params);
    }


    /**
     * 衍生品杠杆与保证金
     *
     * @return
     */
    @PostMapping(value = "/fund-derivative-leverage-margin")
    public ResultVo<Map<String, Object>> fundDerivativeLeverageMargin(@RequestBody Map<String, Object> params) {
        return fundFuturesOptionService.fundDerivativeLeverageMargin(params);
    }


    /**
     * 边际风险贡献
     *
     * @param params
     * @return
     */
    @PostMapping(value = "/fund-marginal-risk-contribution")
    public ResultVo<Map<String, Object>> getFutureRiskCont(@RequestBody MarginalRiskContributionReq params) {
        return ResultVo.ok(this.future.riskCont(params));
    }

    /**
     * @return
     */
    @PostMapping(value = "/fund-futures-option-view")
    public ResultVo<Map<String, List<String>>> fundFuturesOptionView(@RequestBody BaseReq.DefaultReq req) {
        return fundFuturesOptionService.fundFuturesOptionView();
    }
}
